/*
 * Licensed Materials - Use restricted, please refer to the "Samples Gallery" terms
 * and conditions in the IBM International Program License Agreement.
 *
 *  Copyright IBM Corporation 2003, 2004. All Rights Reserved. 
 */
package com.ibm.support;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.uml2.uml.Model;

import com.ibm.xtools.modeler.ui.UMLModeler;
import com.ibm.xtools.pluglets.Pluglet;

public class AbsorbAllFragments extends Pluglet 
{

	/**
	 * Parses the selected model or package and silently absorb its fragments
	 */
	public void plugletmain(String[] args) 
	{

        /* Perform remaining work within a Runnable */
        try 
        {
            UMLModeler.getEditingDomain().runExclusive(new Runnable() 
            {

                public void run() 
                {
                    // Get selection
                    final List elements = UMLModeler.getUMLUIHelper().getSelectedElements();

                    if (elements.size() != 1) 
                    {
						out.println("Cannot perform enumeration on current selection.\nPlease select only one UML Model or Package from the Project Explorer"); //$NON-NLS-1$
						return;
                    }
                    if (!(elements.get(0) instanceof org.eclipse.uml2.uml.Package))
                    {
                    	out.println("Please select a UML model or package only");
                    	return;
                    }
                    
                    org.eclipse.uml2.uml.Package p = (org.eclipse.uml2.uml.Package)elements.get(0);
                    Resource baseResource = p.eResource();                    
                    
                    try
                    {
                        recursivelyRemoveFragments(p, baseResource, p.getModel());
                        out.println("Successfull operation");
                    }
                    catch(Exception e)
                    {
                    	out.println("Exception " + e);
                    }
                }

            });
        } 
        catch (InterruptedException e) 
        {
			out.println("The operation was interrupted"); //$NON-NLS-1$
        }
	}


	/**
	 * Recursively parses the given object to absorb its fragments.
	 * @param object the object to parse
	 * @param baseResource the resource of the selected package / model
	 * @param m the model associated with the selection
	 * @throws Exception
	 */
	private void recursivelyRemoveFragments(Object object, Resource baseResource, Model m) throws Exception 
	{
		if (object == null)
			return;
		
		if (object instanceof EObject) 
		{
			EObject eobject = ((EObject)object);
			
			// Safety checks - shouldn't be necessary except on broken models.
			if (eobject.eResource() == null)
			{
				throw new Exception("eobject.eResource() == null");
			}
			else if (eobject.eResource().getURI() == null)
			{
				throw new Exception("eobject.eResource().getURI() == null");
			}
			else if (eobject.eResource() != baseResource)
			{
				String fragment = eobject.eResource().getURI().toString();
				UMLModeler.absorbFragmentSilently(eobject.eResource(), true);
				UMLModeler.saveModel(m);
				out.println("Fragment " + fragment + " absorbed");
			}
			Iterator contents = ((EObject)object).eContents().iterator();
			while (contents.hasNext()) 
			{
				recursivelyRemoveFragments(contents.next(), baseResource, m);
			}
		}
	}

}
